@@ -1,5 +1,5 @@
 /* poppler-link.cc: qt interface to poppler
- * Copyright (C) 2006-2007, 2013, 2016, Albert Astals Cid
+ * Copyright (C) 2006-2007, 2013, 2016, 2017, Albert Astals Cid
  * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
  * Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
  * Copyright (C) 2012, Tobias Koenig <tokoe@kdab.com>
@@ -34,7 +34,7 @@
 #include "Link.h"
 #include "Rendition.h"
 
-bool operator==( const Ref &r1, const Ref &r2 )
+static bool operator==( const Ref &r1, const Ref &r2 )
 {
 	return r1.num == r2.num && r1.gen == r2.gen;
 }
@@ -296,17 +296,19 @@ class LinkMoviePrivate : public LinkPrivate
 	LinkDestination::LinkDestination(const QString &description)
 		: d( new LinkDestinationPrivate )
 	{
-		QStringList tokens = description.split( ';' );
-		d->kind = static_cast<Kind>(tokens.at(0).toInt());
-		d->pageNum = tokens.at(1).toInt();
-		d->left = tokens.at(2).toDouble();
-		d->bottom = tokens.at(3).toDouble();
-		d->right = tokens.at(4).toDouble();
-		d->top = tokens.at(5).toDouble();
-		d->zoom = tokens.at(6).toDouble();
-		d->changeLeft = static_cast<bool>(tokens.at(7).toInt());
-		d->changeTop = static_cast<bool>(tokens.at(8).toInt());
-		d->changeZoom = static_cast<bool>(tokens.at(9).toInt());
+		const QStringList tokens = description.split( ';' );
+		if (tokens.size() >= 10) {
+			d->kind = static_cast<Kind>(tokens.at(0).toInt());
+			d->pageNum = tokens.at(1).toInt();
+			d->left = tokens.at(2).toDouble();
+			d->bottom = tokens.at(3).toDouble();
+			d->right = tokens.at(4).toDouble();
+			d->top = tokens.at(5).toDouble();
+			d->zoom = tokens.at(6).toDouble();
+			d->changeLeft = static_cast<bool>(tokens.at(7).toInt());
+			d->changeTop = static_cast<bool>(tokens.at(8).toInt());
+			d->changeZoom = static_cast<bool>(tokens.at(9).toInt());
+		}
 	}
 	
 	LinkDestination::LinkDestination(const LinkDestination &other)
